'''
Futtatás a terminálban:
*python* *scriptnév* *kép neve* *xméret* *yméret*

python SM_calc.py mm_logo_fele.png 1. 1.85
'''

import sys # Terminállal kommunikációhoz szükséges függvénycsomag
import numpy as np # numerikus matematika package
import skimage # képfeldolgozáshoz csomaggyűjtemény
from skimage import io # képfeldolgozás: input-output
import skimage.filters as skf # képfeldolgozás: szűrők
from skimage.color import * # képfeldolgozás: színek

np.set_printoptions(precision=4)

# kép binarizálása
def binarise_section(image):
    image_gray=rgb2grey(image)
    thresh=skf.threshold_mean(image_gray)
    return image_gray > thresh

# Keresztmetszet tulajdonságokat számító osztály
class Section_Calculator:
    # Tulajdonságok inicializása
    rs=np.array([0.,0.])
    A = 0.
    I_xy = np.zeros((2,2))
    
    # Keresztmetszet mentén integrátor (pixelek középpontjához rendelve)
    def integrate_over_section(self,fun):
        result = fun(self.pxl_dim[0],self.pxl_dim[1])*\
        sum(sum(fun(x+0.5,y+0.5) for (x,pxl) in enumerate(row) if pxl)\
           for (y,row) in enumerate(self.binary))*\
        self.pxl_dim[0]*self.pxl_dim[1]
        return result
    
    # Keresztmetszet területe
    def calculate_area(self):
        self.A = self.integrate_over_section(lambda x,y: 1)

    # Keresztmetszet súlypontja
    def calculate_CoM(self):
        self.rs[0] = self.integrate_over_section(lambda x,y: x)/self.A
        self.rs[1] = self.integrate_over_section(lambda x,y: y)/self.A
    
    # Keresztmetszet súlypontjához tartozó másodrendű nyomatéki mátrix
    def calculate_I(self):
        Ix = self.integrate_over_section(lambda x,y: y**2)
        Iy = self.integrate_over_section(lambda x,y: x**2)
        Ixy = self.integrate_over_section(lambda x,y: x*y)
        kivonni = self.A*np.array([[self.rs[1]**2,-self.rs[0]*self.rs[1]],\
                   [-self.rs[0]*self.rs[1],self.rs[0]**2]])
        self.I_xy = np.array([[Ix,-Ixy],[-Ixy,Iy]])-kivonni
    
    # Számítások egy függvénybe pakolva
    def calculate_SectionData(self):
        self.calculate_area()
        self.calculate_CoM()
        self.calculate_I()
    
    # Osztály inicializálása fájlnévből, és méretből
    def __init__(self,filename,dimensions):
        # kép beolvasása, fekete-fehérré tétele, binarizálása, 
        self.binary = binarise_section(io.imread(filename))

        # pixelméret kiszámítása
        self.dims = dimensions
        self.pxl_dim = [self.dims[0]/len(self.binary[0]),\
                         self.dims[1]/len(self.binary)]         
        

# sys.argv[0]: scriptnév
# sys.argv[1]: képnév
# sys.argv[2]: x_méret
# sys.argv[3]: y_méret
SC=Section_Calculator(sys.argv[1],[float(n) for n in sys.argv[2:]])
SC.calculate_SectionData()
print('\n----------------------------------')
print('Keresztmetszet területe: %.4f' % SC.A)
print('Keresztmetszet súlypontja [x,y]: ',SC.rs)
print('Másodrendű nyomatéki mátrix: \n',SC.I_xy)
print('Főmásodrendű nyomatékok: ',np.linalg.eig(SC.I_xy)[0])
print('----------------------------------\n')
